﻿Class Window1 
    ' Custom command objects.
    Public Shared CommandImageInvert As RoutedUICommand
    Public Shared CommandImageRed As RoutedUICommand
    Public Shared CommandImageGreen As RoutedUICommand
    Public Shared CommandImageBlue As RoutedUICommand
    Public Shared CommandImageGrayscale As RoutedUICommand
    Public Shared CommandImageAverage As RoutedUICommand

    Private Const offset_a As Integer = 3
    Private Const offset_r As Integer = 2
    Private Const offset_g As Integer = 1
    Private Const offset_b As Integer = 0

    ' Return the ARGB components of a pixel.
    Private Sub GetPixel(ByVal pixels() As Byte, ByVal stride As Integer, ByVal bytes_per_pixel As Integer, ByVal x As Integer, ByVal y As Integer, ByRef a As Byte, ByRef r As Byte, ByRef g As Byte, ByRef b As Byte)
        Dim offset As Integer = y * stride + x * bytes_per_pixel
        a = pixels(offset + offset_a)
        r = pixels(offset + offset_r)
        g = pixels(offset + offset_g)
        b = pixels(offset + offset_b)
    End Sub

    ' Set the ARGB components of a pixel.
    Private Sub SetPixel(ByVal pixels() As Byte, ByVal stride As Integer, ByVal bytes_per_pixel As Integer, ByVal x As Integer, ByVal y As Integer, ByVal a As Byte, ByVal r As Byte, ByVal g As Byte, ByVal b As Byte)
        Dim offset As Integer = y * stride + x * bytes_per_pixel
        pixels(offset + offset_a) = a
        pixels(offset + offset_r) = r
        pixels(offset + offset_g) = g
        pixels(offset + offset_b) = b
    End Sub

    ' Invert the original image.
    Private Sub ExecuteImageInvert(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        ' Get the bitmap source.
        Dim bms_old As BitmapSource = imgOld.Source

        ' Get the size in pixels.
        Dim wid As Integer = bms_old.PixelWidth
        Dim hgt As Integer = bms_old.PixelHeight

        ' Make room for the pixel bytes.
        Dim bytes_per_pixel As Integer = bms_old.Format.BitsPerPixel \ 8
        Dim stride As Integer = wid * bytes_per_pixel
        Dim pixels_old(hgt * stride - 1) As Byte

        ' Get the pixel data.
        Try
            bms_old.CopyPixels(pixels_old, stride, 0)
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try

        ' Make room for the output pixel data.
        Dim pixels_new(hgt * stride - 1) As Byte

        ' Transform the pixel data.
        pixels_old.CopyTo(pixels_new, 0)
        For x As Integer = 0 To wid - 1
            For y As Integer = 0 To hgt - 1
                Dim a As Integer = 0
                Dim r As Integer = 0
                Dim g As Integer = 0
                Dim b As Integer = 0
                GetPixel(pixels_old, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)

                ' Invert.
                r = 255 - r
                g = 255 - g
                b = 255 - b
                SetPixel(pixels_new, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)
            Next y
        Next x

        ' Display the result.
        Dim bms_new As BitmapSource = BitmapSource.Create(wid, hgt, _
            bms_old.DpiX, bms_old.DpiY, bms_old.Format, _
            Nothing, pixels_new, stride)
        imgNew.Source = bms_new
    End Sub

    ' Keep only the red color component.
    Private Sub ExecuteImageRed(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        CopyColors(True, True, False, False)
    End Sub

    ' Keep only the green color component.
    Private Sub ExecuteImageGreen(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        CopyColors(True, False, True, False)
    End Sub

    ' Keep only the blue color component.
    Private Sub ExecuteImageBlue(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        ' MessageBox.Show(btn.Name)
        CopyColors(True, False, False, True)
    End Sub

    ' Copy red, green, or blue color components.
    Private Sub CopyColors(ByVal copy_a As Boolean, ByVal copy_r As Boolean, ByVal copy_g As Boolean, ByVal copy_b As Boolean)
        ' Get the bitmap source.
        Dim bms_old As BitmapSource = imgOld.Source

        ' Get the size in pixels.
        Dim wid As Integer = bms_old.PixelWidth
        Dim hgt As Integer = bms_old.PixelHeight

        ' Make room for the pixel bytes.
        Dim bytes_per_pixel As Integer = bms_old.Format.BitsPerPixel \ 8
        Dim stride As Integer = wid * bytes_per_pixel
        Dim pixels_old(hgt * stride - 1) As Byte

        ' Get the pixel data.
        Try
            bms_old.CopyPixels(pixels_old, stride, 0)
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try

        ' Make room for the output pixel data.
        Dim pixels_new(hgt * stride - 1) As Byte

        ' Transform the pixel data.
        pixels_old.CopyTo(pixels_new, 0)
        For x As Integer = 0 To wid - 1
            For y As Integer = 0 To hgt - 1
                Dim a As Integer = 0
                Dim r As Integer = 0
                Dim g As Integer = 0
                Dim b As Integer = 0
                GetPixel(pixels_old, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)

                ' Save only the selected components.
                If (Not copy_a) Then a = 0
                If (Not copy_r) Then r = 0
                If (Not copy_g) Then g = 0
                If (Not copy_b) Then b = 0

                SetPixel(pixels_new, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)
            Next y
        Next x

        ' Display the result.
        Dim bms_new As BitmapSource = BitmapSource.Create(wid, hgt, _
            bms_old.DpiX, bms_old.DpiY, bms_old.Format, _
            Nothing, pixels_new, stride)
        imgNew.Source = bms_new
    End Sub

    ' Convert to grayscale.
    Private Sub ExecuteImageGrayscale(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        ' Get the bitmap source.
        Dim bms_old As BitmapSource = imgOld.Source

        ' Get the size in pixels.
        Dim wid As Integer = bms_old.PixelWidth
        Dim hgt As Integer = bms_old.PixelHeight

        ' Make room for the pixel bytes.
        Dim bytes_per_pixel As Integer = bms_old.Format.BitsPerPixel \ 8
        Dim stride As Integer = wid * bytes_per_pixel
        Dim pixels_old(hgt * stride - 1) As Byte

        ' Get the pixel data.
        Try
            bms_old.CopyPixels(pixels_old, stride, 0)
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try

        ' Make room for the output pixel data.
        Dim pixels_new(hgt * stride - 1) As Byte

        ' Transform the pixel data.
        pixels_old.CopyTo(pixels_new, 0)
        For x As Integer = 0 To wid - 1
            For y As Integer = 0 To hgt - 1
                Dim a As Integer = 0
                Dim r As Integer = 0
                Dim g As Integer = 0
                Dim b As Integer = 0
                GetPixel(pixels_old, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)

                ' Convert to gray scale.
                Dim gray As Byte = CByte(0.3 * r + 0.5 * g + 0.2 * b)
                SetPixel(pixels_new, stride, bytes_per_pixel, _
                    x, y, a, gray, gray, gray)
            Next y
        Next x

        ' Display the result.
        Dim bms_new As BitmapSource = BitmapSource.Create(wid, hgt, _
            bms_old.DpiX, bms_old.DpiY, bms_old.Format, _
            Nothing, pixels_new, stride)
        imgNew.Source = bms_new
    End Sub

    ' Average the red, green, and blue components.
    Private Sub ExecuteImageAverage(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        ' Get the bitmap source.
        Dim bms_old As BitmapSource = imgOld.Source

        ' Get the size in pixels.
        Dim wid As Integer = bms_old.PixelWidth
        Dim hgt As Integer = bms_old.PixelHeight

        ' Make room for the pixel bytes.
        Dim bytes_per_pixel As Integer = bms_old.Format.BitsPerPixel \ 8
        Dim stride As Integer = wid * bytes_per_pixel
        Dim pixels_old(hgt * stride - 1) As Byte

        ' Get the pixel data.
        Try
            bms_old.CopyPixels(pixels_old, stride, 0)
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try

        ' Make room for the output pixel data.
        Dim pixels_new(hgt * stride - 1) As Byte

        ' Transform the pixel data.
        pixels_old.CopyTo(pixels_new, 0)
        For x As Integer = 0 To wid - 1
            For y As Integer = 0 To hgt - 1
                Dim a As Integer = 0
                Dim r As Integer = 0
                Dim g As Integer = 0
                Dim b As Integer = 0
                GetPixel(pixels_old, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)

                ' Average the components.
                Dim gray As Byte = CByte((r + g + b) / 3)
                SetPixel(pixels_new, stride, bytes_per_pixel, _
                    x, y, a, gray, gray, gray)
            Next y
        Next x

        ' Display the result.
        Dim bms_new As BitmapSource = BitmapSource.Create(wid, hgt, _
            bms_old.DpiX, bms_old.DpiY, bms_old.Format, _
            Nothing, pixels_new, stride)
        imgNew.Source = bms_new
    End Sub

    ' Return True if the CheckBox is checked.
    Private Sub CanExecuteChecked(ByVal sender As Object, ByVal e As CanExecuteRoutedEventArgs)
        e.CanExecute = chkEnable.IsChecked
    End Sub

    ' Make a new window.
    Private Sub NewWindow_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles Button1.Click
        Dim frm As New Window1
        frm.Show()
    End Sub

    ' Prepare custom commands.
    Shared Sub New()
        CommandImageInvert = New RoutedUICommand("_Invert", _
                "Invert Command", GetType(Window1))
        CommandImageInvert.InputGestures.Add( _
                New KeyGesture(Key.I, ModifierKeys.Alt))

        CommandImageRed = New RoutedUICommand("_Red", _
                "Red Command", GetType(Window1))
        CommandImageRed.InputGestures.Add( _
                New KeyGesture(Key.R, ModifierKeys.Alt))

        CommandImageGreen = New RoutedUICommand("_Green", _
                "Green Command", GetType(Window1))
        CommandImageGreen.InputGestures.Add( _
                New KeyGesture(Key.G, ModifierKeys.Alt))

        CommandImageBlue = New RoutedUICommand("_Blue", _
                "Blue Command", GetType(Window1))
        CommandImageBlue.InputGestures.Add( _
                New KeyGesture(Key.B, ModifierKeys.Alt))

        CommandImageGrayscale = New RoutedUICommand("G_rayscale", _
                "Grayscale Command", GetType(Window1))
        CommandImageGrayscale.InputGestures.Add( _
                New KeyGesture(Key.R, ModifierKeys.Alt))

        CommandImageAverage = New RoutedUICommand("_Average", _
                "Average Command", GetType(Window1))
        CommandImageAverage.InputGestures.Add( _
                New KeyGesture(Key.A, ModifierKeys.Alt))
    End Sub
End Class
